#ifndef STASSID
#define STASSID "XXXXX"
#define STAPSK  "YYYYY"
#endif

#define COUNTER_PIN D5
#define START_PIN   D6

#include <TZ.h>

#define MYTZ TZ_America_Los_Angeles
//#define RTC_UTC_TEST 1606654251 // 1510592825 = Monday 13 November 2017 17:07:05 UTC

////////////////////////////////////////////////////////

#include <ESP8266WiFi.h>
#include <coredecls.h>                  // settimeofday_cb()
#include <Schedule.h>
#include <PolledTimeout.h>

#include <time.h>                       // time() ctime()
#include <sys/time.h>                   // struct timeval

#include <sntp.h>                       // sntp_servermode_dhcp()

// for testing purpose:
extern "C" int clock_gettime(clockid_t unused, struct timespec *tp);

////////////////////////////////////////////////////////

static timeval tv;
static timespec tp;
static time_t now;
static uint32_t now_ms, now_us;

static esp8266::polledTimeout::periodicMs showTimeNow(1000);

static bool ntpEnabled = false;
static bool counting   = false;
static bool resetting  = false;

static int currDisp = 0;

static bool twenty4  = true;

void setup() {
  Serial.begin(115200);
  Serial.println("\nStarting...\n");

  pinMode(COUNTER_PIN, OUTPUT);    // pin as output.
  pinMode(START_PIN, OUTPUT);      // pin as output.
  
  // install callback - called when settimeofday is called
  settimeofday_cb(time_is_set_scheduled);

//  time_t rtc = RTC_UTC_TEST;
//  timeval tv = { rtc, 0 };
//  settimeofday(&tv, nullptr);
//  time_is_set_scheduled();

  configTime(MYTZ, "pool.ntp.org");

  // start network
  WiFi.persistent(false);
  WiFi.mode(WIFI_STA);
  WiFi.begin(STASSID, STAPSK);
}

void loop() {
  if (showTimeNow)
    showTime();
}

void time_is_set_scheduled() {
  if (ntpEnabled) return;
  ntpEnabled = true;
  Serial.printf("NTP enabeled\n");
  counterRst();
  ntpEnabled = true;
}


void showTime() {
  if (!ntpEnabled) return;

  // Calc current time integer
  int currTime = calcCurrTime();

  // If we are moving the display already, don't do anything now
  if (resetting || counting) return;
  
  // Find how far display integer is from current time integer
  int diff = currTime - currDisp;
  // diff = 1; // for testing counter trigger
  
  if (diff<0) {  // check for overflow, i.e. 13:00
    counterRst();  // reset counter to zero
    diff = currTime; // needs to catch-up all to match current time, i.e. 1:00
  }
  
  counterAdv(diff);
}


// Current time as an integer
//  1:00:00 ==  10000
//  2:34:20 ==  23420
// 12:00:00 == 120000
int calcCurrTime(){
  now = time(nullptr);
  const tm* tm = localtime(&now);
  byte hr = tm->tm_hour;
  byte mn = tm->tm_min;
  byte sc = tm->tm_sec;
  if (twenty4) {
    if (hr>23) hr = 0;
  } else {
    // Round Hour to 1-12
    if (hr==0) hr = 12;
    if (hr>12) hr -= 12;
  }
  return hr*10000 + mn*100 + sc;
}

// Increment the counter as much as needed to get to current integer (usually one count)
void counterAdv(int d) {
  counting = true;
  if (d>1)
    Serial.printf("Big Diff:  %d\n", d);
  for (int i=0; i<d; i++)
    counterInc();
  if (d>1)
    Serial.printf("Diff Done:  %d\n", d);
  counting = false;
}

static unsigned int delay_mS = 1;
void counterInc() {
  digitalWrite(COUNTER_PIN, HIGH);
  delay(delay_mS);
  digitalWrite(COUNTER_PIN, LOW);
  delay(delay_mS);
  currDisp++;  // Add one to the display
}

static unsigned int reset_mS = 200;
void counterRst() {
  resetting = true;
  Serial.println("Reset Display");
  digitalWrite(START_PIN, HIGH);
  delay(reset_mS);
  digitalWrite(START_PIN, LOW);
  currDisp = 0; // display is now at zero
  resetting = false;
}
